require 'rubygems'
require 'httparty'
require 'digest/md5'
require 'json'

class AvatarsIO
  include HTTParty
  base_uri 'avatars.io'
  format :json
  
  @@client_id = ''
  @@access_token = ''
  
  def self.client_id=(client_id)
    @@client_id = client_id
  end
  
  def self.access_token=(access_token)
    @@access_token = access_token
  end
  
  def self.upload(filename, identifier = '')
    file = File.read filename
    response = post '/v1/token', {
      :headers => {
        'x-client_id' => @@client_id,
        'Authorization' => "OAuth #{ @@access_token }",
        'Content-Type' => 'application/json'
      }, :body => {
        :data => {
          :filename => filename,
          :md5 => Digest::MD5::hexdigest(file),
          :size => File.size(filename),
          :path => identifier
        }
      }.to_json
    }
    
    return response["data"]["url"] if !response["data"]["upload_info"]
    
    info = response["data"]["upload_info"]
    AvatarsIOFile.put info["upload_url"].gsub('http://s3.amazonaws.com', ''), {
      :headers => {
        'Authorization' => info["signature"],
        'Date' => info["date"],
        'Content-Type' => info["content_type"],
        'x-amz-acl' => 'public-read'
      },
      :body => file
    }
    
    response = post "/v1/token/#{ response["data"]["id"] }/complete", {
      :headers => {
        'x-client_id' => @@client_id,
        'Authorization' => "OAuth #{ @@access_token }"
      },
      :body => ''
    }
    response["data"]
  end
  
  def self.avatar_url(service = 'twitter', key)
    "http://avatars.io/#{ service }/#{ key }"
  end
  
  def self.auto_url(key, services = [])
    if services.size.eql? 0
      "http://avatars.io/auto/#{ key }"
    else
      "http://avatars.io/auto/#{ key }?services=#{ services.join ',' }"
    end
  end
end

def avatar_url(service, key)
  AvatarsIO.avatar_url service, key
end

def auto_avatar_url(key, services = [])
  AvatarsIO.auto_url key, services
end

class AvatarsIOFile
  include HTTParty
  base_uri 's3.amazonaws.com'
end